;==============================================================================================================
;	Autoit script for An Untitled Story
;				By
;			  Zoonel
;==============================================================================================================
;	The purpose of the script is to 
;	change the Read Only attribute 
;	from the savefiles with an hotkey 
;	or a gamepad button.
;	The AUSreadmode.ini file is for 
;	the script config.
;
;==============================================================================================================
;   AUSreadmode.ini options:
;   -----------------------
;  [global]
;  GamePath      => path to the game folder i.g.: D:\AnUntitledStory
;  UseGamepad    => 1 or 0, 1 to enable gamepad section
;  GamepadID     => n , number of the gamepad ID. 0 for default gamepad
;  
;  [saves]
;  Save1         => 1 or 0, 1 to  authorize the change of attribute for the specific savefile
;  Save2         => idem
;  Save3         => idem
;
;  [keyboard]
;  Quit          => keyboard hotkey to quit the script, only aphanumeric characters or blank
;  ReadOnlyON    => keyboard hotkey to enable the readonly attribute, only aphanumeric characters or blank
;  ReadOnlyOFF   => keyboard hotkey to disable the readonly attribute, only aphanumeric characters or blank
;
;                n.b. : you can use the autoit ! + ^ # special keys for ALT SHIFT CTRL Windowskey
;                       i.g.: Quit=+!q  for SHIFT+ALT+q
;
;  [gamepad]
;  Quit          => gamepad button to quit the script, a number from 1 to 12 or blank
;  ReadOnlyON    => gamepad button to enable the readonly attribute, a number from 1 to 12 or blank
;  ReadOnlyOFF   => gamepad button to disable the readonly attribute, a number from 1 to 12 or blank
;
;
;
;==============================================================================================================
;	rem:  to use the readonly trick you must first set a savefile to readonly, then continue the corresponding
;         game and then directly quit to the menu. An error screen will be prompted, answer 'abort'.
;		  You can now freely switch the readonly attrib to on/off and still beeing able to die or
;         quit and restart from last save without weird bugs or errors.
;         Do this each time you launch the game executable.
;==============================================================================================================
#include "GamePadEx.au3" ;Shafayat UDFs for gamepad handling

;---------------------------------------------------------------------------------------------------------------
; file definitions
;---------------------------------------------------------------------------------------------------------------
$iniFile = @ScriptDir & "\AUSreadmode.ini"
$GameSave1 = "UntitledSave1"
$GameSave2 = "UntitledSave2"
$GameSave3 = "UntitledSave3"
$PathCheck = "BGs"

;---------------------------------------------------------------------------------------------------------------
; config file parsing (.ini file)
;---------------------------------------------------------------------------------------------------------------
If FileExists($iniFile) Then
	$IniErrorMsg = ""
	
	;[global] section
	$GamePath = IniRead ( $iniFile, "global", "GamePath", "D:\AnUntitledStory" )
	$UseGamepad = IniRead ( $iniFile, "global", "UseGamepad", "0" )
	$GamepadID = IniRead ( $iniFile, "global", "GamepadID", "0" )
	If Not FileExists( $GamePath & "\" & $PathCheck ) Then   ;detect if we are in the correct folder
		$IniErrorMsg &= "GamePath is incorrect : " & $GamePath & @CRLF
	EndIf
	If $UseGamepad <> "0" And $UseGamepad <> "1" Then
		$IniErrorMsg &= "UseGamepad must be 0 or 1 not : " & $UseGamepad & @CRLF
	EndIf
	If Not StringIsInt($GamepadID) Then
		$IniErrorMsg &= "GamepadID must be an integer not : " & $GamepadID & @CRLF
	EndIf
	$UseGamepad = int( $UseGamepad )
	$GamepadID = int( $GamepadID )
	
	;[saves] section
	$Save1 = IniRead ( $iniFile, "saves", "Save1", "0" )
	$Save2 = IniRead ( $iniFile, "saves", "Save2", "0" )
	$Save3 = IniRead ( $iniFile, "saves", "Save3", "0" )
	If $Save1 <> "0" And $Save1 <> "1" Then
		$IniErrorMsg &= "Save1 must be 0 or 1 not : " & $Save1 & @CRLF
	EndIf
	If $Save2 <> "0" And $Save2 <> "1" Then
		$IniErrorMsg &= "Save2 must be 0 or 1 not : " & $Save2 & @CRLF
	EndIf
	If $Save3 <> "0" And $Save1 <> "1" Then
		$IniErrorMsg &= "Save3 must be 0 or 1 not : " & $Save3 & @CRLF
	EndIf
	$Save1 = int( $Save1 )
	$Save2 = int( $Save2 )
	$Save3 = int( $Save3 )

	;[keyboard] section
	$KBQuit = IniRead ( $iniFile, "keyboard", "Quit", "" )
	$KBReadOnlyON = IniRead ( $iniFile, "keyboard", "ReadOnlyON", "" )
	$KBReadOnlyOFF = IniRead ( $iniFile, "keyboard", "ReadOnlyOFF", "" )
	      ;\h* stands for empty or any blank horizontal space
	If StringRegExp($KBQuit , "^(\h*|[!+^#]{0,3}[[:alnum:]])$" , 0) <> 1 Then
		$IniErrorMsg &= "KBQuit is not correct : " & $KBQuit & @CRLF
	ElseIf StringRegExp($KBQuit , "^\h*$" , 0) <> 1 Then
		If Not HotKeySet($KBQuit, "_Quit") Then
			$IniErrorMsg &= "KBQuit cannot be a hotkey : " & $KBQuit & @CRLF
		EndIf
	EndIf
	If StringRegExp($KBReadOnlyON , "^(\h*|[!+^#]{0,3}[[:alnum:]])$" , 0) <> 1 Then
		$IniErrorMsg &= "KBReadOnlyON is not correct : " & $KBReadOnlyON & @CRLF
	ElseIf StringRegExp($KBReadOnlyON , "^\h*$" , 0) <> 1 Then
		If Not HotKeySet($KBReadOnlyON, "_ReadOnlyON") Then
			$IniErrorMsg &= "KBReadOnlyON cannot be a hotkey : " & $KBReadOnlyON & @CRLF
		EndIf
	EndIf
	If StringRegExp($KBReadOnlyOFF , "^(\h*|[!+^#]{0,3}[[:alnum:]])$" , 0) <> 1 Then
		$IniErrorMsg &= "KBReadOnlyOFF is not correct : " & $KBReadOnlyOFF & @CRLF
	ElseIf StringRegExp($KBReadOnlyOFF , "^\h*$" , 0) <> 1 Then
		If Not HotKeySet($KBReadOnlyOFF, "_ReadOnlyOFF") Then
			$IniErrorMsg &= "KBReadOnlyOFF cannot be a hotkey : " & $KBReadOnlyOFF & @CRLF
		EndIf 
	EndIf
	
	;[gamepad] section
	$GPQuit = IniRead ( $iniFile, "gamepad", "Quit", "" )
	$GPReadOnlyON = IniRead ( $iniFile, "gamepad", "ReadOnlyON", "" )
	$GPReadOnlyOFF = IniRead ( $iniFile, "gamepad", "ReadOnlyOFF", "" )
	       ;\h* stands for empty or any blank horizontal space
	If StringRegExp($GPQuit , "^(\h*|[1-9]|1[012])$" , 0) <> 1 Then
		$IniErrorMsg &= "GPQuit is not correct : " & $GPQuit & @CRLF
	EndIf
	If StringRegExp($GPReadOnlyON , "^(\h*|[1-9]|1[012])$" , 0) <> 1 Then
		$IniErrorMsg &= "GPReadOnlyON is not correct : " & $GPReadOnlyON & @CRLF
	EndIf
	If StringRegExp($GPReadOnlyOFF , "^(\h*|[1-9]|1[012])$" , 0) <> 1 Then
		$IniErrorMsg &= "GPReadOnlyOFF is not correct : " & $GPReadOnlyOFF & @CRLF
	EndIf
	
	; error handling
    If $IniErrorMsg <> "" Then
		MsgBox(0x40,"AUSreadmode.ini errors", $IniErrorMsg & @LF & @LF & "Please correct AUSreadmode.ini before restarting the application")
		Exit
	EndIf
Else
	;no file found, rewrite a clean .ini config file
    IniWriteSection($iniFile,"global", "GamePath=C:\path\to\an untitled story" & @LF & "UseGamepad=0" & @LF & "GamepadID=0")
	IniWriteSection($iniFile,"saves", "Save1=1" & @LF & "Save2=0" & @LF & "Save3=0")
	IniWriteSection($iniFile,"keyboard", "Quit=q" & @LF & "ReadOnlyON=k" & @LF & "ReadOnlyOFF=l")
	IniWriteSection($iniFile,"gamepad", "Quit=" & @LF & "ReadOnlyON=5" & @LF & "ReadOnlyOFF=6")
	MsgBox(0x40,"AUSreadmode", "AUSreadmode.ini has been created" & @LF & "Please modify it before restarting the application")
	Exit
EndIf

;---------------------------------------------------------------------------------------------------------------
; Gamepad initialization
;---------------------------------------------------------------------------------------------------------------
If $UseGamepad Then
	Global $myGamePadHandle = GPExInitialize() ; Get a handle. This doesn't have to be global but needs to be passed to all the other functions.
	Global $myID = $GamepadID ; We want to monitor the GamePad with tha specific ID
	If StringRegExp($GPQuit , "^\h*$" , 0) <> 1 Then
		$GPQuit = GPExButton(Int( $GPQuit ))
	Else
		$GPQuit = 0 ;if the button number is left blank we use 0
	EndIf
	If StringRegExp($GPReadOnlyON , "^\h*$" , 0) <> 1 Then
		$GPReadOnlyON = GPExButton(Int( $GPReadOnlyON ))
	Else
		$GPReadOnlyON = 0
	EndIf
	If StringRegExp($GPReadOnlyOFF , "^\h*$" , 0) <> 1 Then
		$GPReadOnlyOFF = GPExButton(Int( $GPReadOnlyOFF ))
	Else
		$GPReadOnlyOFF = 0
	EndIf
EndIf

;---------------------------------------------------------------------------------------------------------------
; Main script loop
;---------------------------------------------------------------------------------------------------------------
While 1
    Sleep(20)
	If $UseGamepad Then
		Select
			Case BitAND(GPExGetPressed($myGamePadHandle, $myID), $GPReadOnlyON)
				_ReadOnlyON()
			Case BitAND(GPExGetPressed($myGamePadHandle, $myID), $GPReadOnlyOFF)
				_ReadOnlyOFF()
			Case BitAND(GPExGetPressed($myGamePadHandle, $myID), $GPQuit)
				_Quit()
		EndSelect
	EndIf
WEnd

;---------------------------------------------------------------------------------------------------------------
; script functions
;---------------------------------------------------------------------------------------------------------------
Func _ReadOnlyON()
	IF $Save1 Then FileSetAttrib( $GamePath & "\" & $GameSave1 , "+R")
	IF $Save2 Then FileSetAttrib( $GamePath & "\" & $GameSave2 , "+R")
	IF $Save3 Then FileSetAttrib( $GamePath & "\" & $GameSave3 , "+R")
EndFunc 

Func _ReadOnlyOFF()
	IF $Save1 Then FileSetAttrib( $GamePath & "\" & $GameSave1 , "-R")
	IF $Save2 Then FileSetAttrib( $GamePath & "\" & $GameSave2 , "-R")
	IF $Save3 Then FileSetAttrib( $GamePath & "\" & $GameSave3 , "-R")
EndFunc 

Func _Quit()
	If $UseGamepad Then
		GPExTerminate($myGamePadHandle) ; Just for good manners.
	EndIf
	Exit
EndFunc   ;==>_Quit